export class ImSocket {
  /**
   * @param {string} url - WebSocket 服务器地址
   * @param {number} checkInterval - 检查连接状态的时间间隔，单位毫秒
   * @param {number} reconnectDelay - 连接断开后重连的延迟，单位毫秒
   */
  constructor(url, checkInterval = 5000, reconnectDelay = 3000) {
    this.url = url;
    this.checkInterval = checkInterval;
    this.reconnectDelay = reconnectDelay;
    this.ws = null;
    this.isManualClose = false;
    this.connect();
    this.startHeartbeat();
    this.onMessageCallback = null;
  }

  connect() {
    this.ws = new WebSocket(this.url);

    // 绑定事件
    this.ws.onopen = (event) => {};

    this.ws.onmessage = (event) => {
      // 根据需要处理消息
      if (this.onMessageCallback) this.onMessageCallback(event.data);
    };

    this.ws.onerror = (error) => {};

    this.ws.onclose = (event) => {
      // 如果不是主动关闭，则重连
      if (!this.isManualClose) {
        setTimeout(() => this.reconnect(), this.reconnectDelay);
      }
    };
  }

  reconnect() {
    this.connect();
  }

  // 定时检查连接状态
  startHeartbeat() {
    this.heartbeatTimer = setInterval(() => {
      if (!this.ws || this.ws.readyState !== WebSocket.OPEN) {
        this.reconnect();
      }
    }, this.checkInterval);
  }

  // 主动关闭 WebSocket 连接，并清除定时任务
  close() {
    this.isManualClose = true;
    if (this.heartbeatTimer) {
      clearInterval(this.heartbeatTimer);
    }
    if (this.ws) {
      this.ws.close();
    }
  }
  onMessage(callback) {
    this.onMessageCallback = callback;
  }
}
